/*****************************************************
This program was produced by the
CodeWizardAVR V1.24.8d Professional
Automatic Program Generator
 Copyright 1998-2006 Pavel Haiduc, HP InfoTech s.r.l.
http://www.hpinfotech.com

Project : 
Version : 
Date    : 24.04.2007
Author  : F4CG                            
Company : F4CG                            
Comments: 


Chip type           : ATmega32
Program type        : Application
Clock frequency     : 4,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 512
*****************************************************/
/*
   rc5dec.c 0.2.0 - Interrupt driven RC5 decoder for Atmel AVR C's.
   Website: http://markh.de/rc5dec/
   Copyright (C) 2003  Mark Haemmerling <rc5@markh.de>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Preconfigured for INT1 (PD3) and Timer/Counter0.
*/

#include <mega32.h>
#include <io.h>
#include <m8_128.h>
#include <stdio.h>
#include <delay.h>



// set your clock speed here
#define XTAL 4000000

// you may have to experiment a little with this
// in case of heavy disturbance, try values around 60
// try to use odd values (for symmetrical level detection)
#define RC5RESAMPLE 41  

// this autoconfigures all timer values:
#if XTAL < 285000
#define PRESCALEDIV 1
#define TIMERCR (1<<CS00)
#elif XTAL < 2300000
#define PRESCALEDIV 8
#define TIMERCR (1<<CS01)
#elif XTAL < 18000000
#define PRESCALEDIV 64
#define TIMERCR (1<<CS01) | (1<<CS00)
#else
#define PRESCALEDIV 256
#define TIMERCR (1<<CS02)
#endif
#define RC5TIMERFIRST 0x100-((unsigned char) (XTAL/PRESCALEDIV*0.0008890))
#define RC5TIMERSECOND 0x100-((unsigned char) (XTAL/PRESCALEDIV*0.0004445))
#define RC5TIMERCANCEL 0x100-((unsigned char) (XTAL/PRESCALEDIV*0.0008890))

volatile unsigned int rc5_code = 0;
volatile unsigned char rc5_first_half;

interrupt [EXT_INT1] void ext_int1_isr(void)
{
 /*	external interrupt handler
	edge from IR receiver detected.
	this is assumed to be the middle of a bit.
*/
	unsigned char i;
	// resample to filter out spikes
   	if (MCUCR & (1<<ISC10)) {
		// we were waiting for a rising edge, so cancel if we sample low
		for (i = RC5RESAMPLE; i; i--) if (!(PIND & (1<<PIND3))) return;
	} else {
		// vice versa...
		for (i = RC5RESAMPLE; i; i--) if ((PIND & (1<<PIND3))) return;
	}   
        TCNT0 = RC5TIMERSECOND; // preset timer to sample (ovf int) at second half (3/4 bit time)
	rc5_first_half = 0; // next sample will be in the second half
        TIMSK |=(1<<TOIE0); // enable timer ovf int
	GICR &=~(1<<INT1); // disable ext int (this handler)  
}

interrupt [TIM0_OVF] void timer0_ovf_isr(void)
/*	timer overflow handler
	sample the present level of the interrupt line.
*/	
 {	static unsigned int rc5_shift, rc5_temp;
	static unsigned char rc5_bit = 0, level, first_level = 1;
	unsigned char i = 0, temp;
        PORTB.0^=1;
	// resample to filter out spikes
	for (temp = RC5RESAMPLE; temp; temp--) if (PIND & (1<<PIND3)) i++;
	level = (i > ((unsigned char) (RC5RESAMPLE/2)));   

	if (rc5_first_half == 2) {
		// this is a timeout, cancel operation
		goto rc5_cancel;
	} else if (rc5_first_half == 1) {
		// this sample is taken in the first half of the bit (1/4 bit time)
		first_level = level; // save current level
		if (level) MCUCR &=~(1<<ISC10); // currently at high level, wait for falling edge
		else MCUCR |=(1<<ISC10); // currently at low level, wait for rising edge
		rc5_first_half = 2; // special code, means: next ovf int is a timeout
		GICR |=(1<<INT1); // much better: wait for edge ;)
		TCNT0 = RC5TIMERCANCEL; // at timer ovf (in around 1/2 bit time) we should get a timeout
	} else {
		// this sample is taken in the second half of the bit (3/4 bit time)
		TCNT0 = RC5TIMERFIRST; // next sample at first half (1/4 bit time) of next bit
		rc5_first_half = 1;
		if (first_level != level) {
			// levels differ -> valid manchester encoded bit detected
			if (!rc5_bit) {
				// this is the first bit, reset values
				rc5_temp = 0;
				rc5_shift = 0x8000;
			}
			if (!level) rc5_temp |= rc5_shift; // low level (low-active) so store a '1'
			else if (rc5_bit < 2) goto rc5_cancel; // high level ('0' bit) as startbit -> error
			if (++rc5_bit == 14) {
				// reception complete, stop operation
				rc5_code = rc5_temp; 
				goto rc5_cancel;
			}
			if (rc5_shift == 0x0100) rc5_shift = 0x0020;
			else rc5_shift >>= 1;
		} else {
			// error in manchester stream -> cancel operation 
			goto rc5_cancel;
		}
	}
	return;

rc5_cancel:
	MCUCR &=~(1<<ISC10); // falling edge
	GICR |=(1<<INT1); // ext. int. enabled
	GIFR &=~(1<<INTF1); // clear an eventually set ext. int. flag
	TIMSK &=~(1<<TOIE0); // disable timer int.
	first_level = 1; // as the next ext. int. appears at the falling edge, assume the first level as high
	rc5_bit = 0;
}


void rc5_init(void){
/*	init for ext. int. and timer.
	call this once to activate the rc5 decoder.
*/
	TCCR0 = TIMERCR;
	DDRD &=~(1<<PD3); // pin is input
	PORTD &=~(1<<PD3); // pullup disabled (receiver has defined output)
	TIMSK &=~(1<<TOIE0); // timer ovf disabled. gets enabled by ext. int.
	MCUCR |=(1<<ISC11); // ext. int. activated by falling edge
	MCUCR &=~(1<<ISC10);
	GIFR = (1<<INTF1); // clear ext. int. flag
	GICR |=(1<<INT1); // enable ext. int.
}

// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
{
// Place your code here

}


// Declare your global variables here

void main(void)
{
// Declare your local variables here
char rc5_command,rc5_system;
// Input/Output Ports initialization
// Port A initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTA=0x00;
DDRA=0x00;

// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTB=0x00;
DDRB=0x01;
// Port C initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTD=0x00;
DDRD=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Low level
// INT1: Off
// INT2: Off
GICR|=0x40;
MCUCR=0x00;
MCUCSR=0x00;
GIFR=0x40;


// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: Off
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud rate: 19200
UCSRA=0x00;
UCSRB=0x08;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x0C;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// Global enable interrupts
#asm("sei");
rc5_init(); 
while (1) {
      if (rc5_code) {
      rc5_command=(unsigned char) rc5_code;
      rc5_system=(unsigned char) (rc5_code>>6);
      rc5_system&=0b00011111;
      printf("System = %i ",rc5_system);
      putsf("\r\n");
      printf("Command = %i ",rc5_command);
      putsf("\r\n");
      rc5_code = 0; // clear code
      rc5_command = 0;
      delay_ms(100);
     };	
	
}
}
